# Implements the BinectApi
class BinectApi
    include ActiveModel::Model

    # Accessor
    attr_accessor :login,
        :password,
        :is_error,
        :error,
        :fault_code,
        :api_wsdl_url


    validates :login,       presence: true #, length: {in:2..255}
    validates :password,    presence: true

    # Constructor
    # @param [Hash] hash It must contain :login and :password
    def initialize(hash)
        super(hash)
        self.is_error = false

        @client = Savon.client(wsdl: hash[:api_wsdl_url] || "https://app.binect.de/ws/services/lettershop?wsdl", basic_auth: [ hash[:login], hash[:password] ])
    end

    # Return the available balance
    # Question: how to convert the balance to €?
    # @return [HASH] with the given keys :whole_balance, :real_balance and :action_balance=>"500"
    def balance
        response = @client.call(:get_balance)
        self.is_error = false
        response.body[:get_balance_response][:balance]
    end

    # Upload file to server.
    # The method will immediately upload the file. It returns the order_id if the document validation succeeds. Otherwise
    # is_error will be true and the fault_code is returned
    # @param [String] filename
    # @param [Object] file_data binary file that should be uploaded
    # @return [String] order_id or fault_code
    def upload(filename, file_data)
        enc_file_data   = Base64.encode64(file_data)
        message         = { filename: filename, fileContent: enc_file_data, parentOrderID: 0 }
        begin
            response        = @client.call(:upload, message: message)
            self.is_error = false
            response.body[:upload_response][:order_id]
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:document_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            self.fault_code
        end
    end

    # Delete a previous stored order
    # @param order_id [String] the Id of the order to delete. @see upload
    # @return [bool] true or false if error occured
    def delete_order(order_id)
        begin
            @client.call(:delete_order, message: { orderID: order_id } )
            self.is_error = false
            true
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            false
        end
    end

    # Cancel an order in process
    # @param order_id [String] the Id of the order to cancel. @see upload
    # @return [bool] true or false if error occured
    def cancel_order(order_id)
        begin
            @client.call(:cancel_order, message: { orderID: order_id } )
            self.is_error = false
            true
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            false
        end
    end

    # Send a list of orders
    # @param order_ids [Array<String>] the Ids of the orders to send. @see upload
    # @return [bool] true or false if error occured
    # DISABLED! The server always returns that sendOrders not exist.
    def send_orders(order_ids)
        fail "not yet working"
        begin
            @client.call(:send_orders, message: { orderIDs: order_ids } )
            self.is_error = false
            true
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            false
        end
    end

    # Send a whole shopping cart
    # @param order_id [String] the Id of the order to cancel. @see upload
    # @return [bool] true or false if error occured
    def send_whole_shopping_cart
        begin
            @client.call(:send_whole_shopping_cart)
            self.is_error = false
            true
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            false
        end
    end
end
